@@ -28,6 +28,7 @@
 #include <Rcpp.h>
 #include <Rcpp/unwindProtect.h>
 #include "readsparse.h"
+#include "utils.h"
 // [[Rcpp::plugins(cpp11)]]
 // [[Rcpp::plugins(unwindProtect)]]
 
@@ -37,11 +38,13 @@ extern "C" {
     FILE *RC_fopen(const SEXP fn, const char *mode, const Rboolean expand);
 }
 #define R_fopen RC_fopen
+#define SUPPORTS_NON_ASCII true
 
 #else
 #   if defined(_WIN32) || defined(_WIN64)
 /* https://stackoverflow.com/questions/2573834/c-convert-string-or-char-to-wstring-or-wchar-t */
 /*  */
+#       if (defined(__GNUC__) && (__GNUC__ < 5))
 #include <locale>
 #include <codecvt>
 #include <string>
@@ -54,16 +57,29 @@ FILE *CRAN_acceptable_fopen(Rcpp::CharacterVector fname, const char *mode, bool
     std::wstring wide = converter.from_bytes(s.get_cstring());
     std::string mode__(mode);
     std::wstring mode_ = converter.from_bytes(mode__);
-    return _wfopen( wide.c_str(), mode_.c_str());
+    return _wfopen(wide.c_str(), mode_.c_str());
 }
+#define SUPPORTS_NON_ASCII true
+#       else
+FILE *CRAN_acceptable_fopen(Rcpp::CharacterVector fname, const char *mode, bool expand)
+{
+    if (expand)
+        fname = Rcpp::Function("path.expand")(fname);
+    Rcpp::String s(fname[0]);
+    return fopen(s.get_cstring(), mode);
+}
+#define SUPPORTS_NON_ASCII false
+#       endif
 #   else
 FILE *CRAN_acceptable_fopen(Rcpp::CharacterVector fname, const char *mode, bool expand)
 {
     if (expand)
         fname = Rcpp::Function("path.expand")(fname);
-    std::string s = Rcpp::as<std::string>(fname);
-    return fopen(s.c_str(), mode);
+    Rcpp::String s(fname[0]);
+    s.set_encoding(CE_NATIVE);
+    return fopen(s.get_cstring(), mode);
 }
+#define SUPPORTS_NON_ASCII true
 #   endif
 FILE *R_fopen(const SEXP fname, const char *mode, const Rboolean expand)
 {
@@ -71,6 +87,38 @@ FILE *R_fopen(const SEXP fname, const char *mode, const Rboolean expand)
 }
 #endif
 
+// [[Rcpp::export(rng = false)]]
+bool supports_nonascii_internal()
+{
+    return SUPPORTS_NON_ASCII;
+}
+
+class FileOpener
+{
+public:
+    FILE *handle = NULL;
+    FileOpener(const SEXP fname, const char *mode, const Rboolean expand)
+    {
+        if (this->handle != NULL)
+            this->close_file();
+        this->handle = R_fopen(fname, mode, expand);
+    }
+    FILE *get_handle()
+    {
+        return this->handle;
+    }
+    void close_file()
+    {
+        if (this->handle != NULL) {
+            fclose(this->handle);
+            this->handle = NULL;
+        }
+    }
+    ~FileOpener()
+    {
+        this->close_file();
+    }
+};
 
 SEXP convert_IntVecToRcpp(void *data)
 {
@@ -89,6 +137,48 @@ SEXP convert_StringStreamToRcpp(void *data)
     return Rcpp::wrap<Rcpp::CharacterVector>(((std::stringstream*)data)->str());
 }
 
+void sort_sparse_indices_known_ncol
+(
+    Rcpp::IntegerVector indptr,
+    Rcpp::IntegerVector indices,
+    Rcpp::NumericVector values,
+    int ncol
+)
+{
+    if (values.size())
+        sort_sparse_indices_known_ncol<int, double>(
+            INTEGER(indptr),
+            INTEGER(indices),
+            REAL(values),
+            indptr.size()-1, ncol
+        );
+    else
+        sort_sparse_indices_known_ncol<int>(
+            INTEGER(indptr),
+            INTEGER(indices),
+            indptr.size()-1, ncol
+        );
+}
+
+// [[Rcpp::export(rng = false)]]
+Rcpp::NumericVector deepcopy_num(Rcpp::NumericVector x)
+{
+    return Rcpp::NumericVector(x.begin(), x.end());
+}
+
+// [[Rcpp::export(rng = false)]]
+Rcpp::IntegerVector deepcopy_int(Rcpp::IntegerVector x)
+{
+    return Rcpp::IntegerVector(x.begin(), x.end());
+}
+
+// [[Rcpp::export(rng = false)]]
+Rcpp::LogicalVector deepcopy_log(Rcpp::LogicalVector x)
+{
+    return Rcpp::LogicalVector(x.begin(), x.end());
+}
+
+
 // [[Rcpp::export(rng = false)]]
 Rcpp::List read_multi_label_R
 (
@@ -116,7 +206,8 @@ Rcpp::List read_multi_label_R
     std::vector<int> qid;
     size_large nrows, ncols, nclasses;
 
-    FILE *input_file = R_fopen(fname[0], "r", TRUE);
+    FileOpener file_(fname[0], "r", TRUE);
+    FILE *input_file = file_.get_handle();
     if (input_file == NULL)
     {
         REprintf("Error %d: %s\n", errno, strerror(errno));
@@ -138,7 +229,7 @@ Rcpp::List read_multi_label_R
         text_is_base1,
         assume_no_qid
     );
-    if (input_file != NULL) fclose(input_file);
+    file_.close_file();
 
     if (!succeeded)
         return Rcpp::List();
@@ -271,7 +362,8 @@ Rcpp::List read_single_label_R
     std::vector<int> qid;
     size_large nrows, ncols, nclasses;
 
-    FILE *input_file = R_fopen(fname[0], "r", TRUE);
+    FileOpener file_(fname[0], "r", TRUE);
+    FILE *input_file = file_.get_handle();
     if (input_file == NULL)
     {
         REprintf("Error %d: %s\n", errno, strerror(errno));
@@ -292,7 +384,7 @@ Rcpp::List read_single_label_R
         text_is_base1,
         assume_no_qid
     );
-    if (input_file != NULL) fclose(input_file);
+    file_.close_file();
     
     if (!succeeded)
         return Rcpp::List();
@@ -414,12 +506,31 @@ bool write_multi_label_R
     const bool append
 )
 {
-    FILE *output_file = R_fopen(fname[0], append? "a" : "w", TRUE);
+    FileOpener file_(fname[0], append? "a" : "w", TRUE);
+    FILE *output_file = file_.get_handle();
     if (output_file == NULL)
     {
         REprintf("Error %d: %s\n", errno, strerror(errno));
         return false;
     }
+
+    if (sort_indices)
+    {
+        sort_sparse_indices_known_ncol(
+            indptr,
+            indices,
+            values,
+            ncols
+        );
+
+        sort_sparse_indices_known_ncol(
+            indptr_lab,
+            indices_lab,
+            Rcpp::NumericVector(),
+            nclasses
+        );
+    }
+
     bool succeeded = write_multi_label(
         output_file,
         INTEGER(indptr),
@@ -434,12 +545,12 @@ bool write_multi_label_R
         ncols,
         nclasses,
         ignore_zero_valued,
-        sort_indices,
+        false,
         text_is_base1,
         add_header,
         decimal_places
     );
-    if (output_file != NULL) fclose(output_file);
+    file_.close_file();
     return succeeded;
 }
 
@@ -462,6 +573,23 @@ Rcpp::List write_multi_label_to_str_R
     const int decimal_places
 )
 {
+    if (sort_indices)
+    {
+        sort_sparse_indices_known_ncol(
+            indptr,
+            indices,
+            values,
+            ncols
+        );
+
+       sort_sparse_indices_known_ncol(
+            indptr_lab,
+            indices_lab,
+            Rcpp::NumericVector(),
+            nclasses
+        );
+    }
+
     Rcpp::List out = Rcpp::List::create(
         Rcpp::_["str"] = R_NilValue
     );
@@ -480,7 +608,7 @@ Rcpp::List write_multi_label_to_str_R
         ncols,
         nclasses,
         ignore_zero_valued,
-        sort_indices,
+        false,
         text_is_base1,
         add_header,
         decimal_places
@@ -511,7 +639,8 @@ bool write_single_label_numeric_R
     const bool append
 )
 {
-    FILE *output_file = R_fopen(fname[0], append? "a" : "w", TRUE);
+    FileOpener file_(fname[0], append? "a" : "w", TRUE);
+    FILE *output_file = file_.get_handle();
     if (output_file == NULL)
     {
         REprintf("Error %d: %s\n", errno, strerror(errno));
@@ -536,7 +665,7 @@ bool write_single_label_numeric_R
         add_header,
         decimal_places
     );
-    if (output_file != NULL) fclose(output_file);
+    file_.close_file();
     return succeeded;
 }
 
@@ -559,12 +688,24 @@ bool write_single_label_integer_R
     const bool append
 )
 {
-    FILE *output_file = R_fopen(fname[0], append? "a" : "w", TRUE);
+    FileOpener file_(fname[0], append? "a" : "w", TRUE);
+    FILE *output_file = file_.get_handle();
     if (output_file == NULL)
     {
         REprintf("Error %d: %s\n", errno, strerror(errno));
         return false;
     }
+
+    if (sort_indices)
+    {
+        sort_sparse_indices_known_ncol(
+            indptr,
+            indices,
+            values,
+            ncols
+        );
+    }
+
     bool succeeded = write_single_label(
         output_file,
         INTEGER(indptr),
@@ -579,12 +720,12 @@ bool write_single_label_integer_R
         ncols,
         nclasses,
         ignore_zero_valued,
-        sort_indices,
+        false,
         text_is_base1,
         add_header,
         decimal_places
     );
-    if (output_file != NULL) fclose(output_file);
+    file_.close_file();
     return succeeded;
 }
 
@@ -605,6 +746,16 @@ Rcpp::List write_single_label_numeric_to_str_R
     const int decimal_places
 )
 {
+    if (sort_indices)
+    {
+        sort_sparse_indices_known_ncol(
+            indptr,
+            indices,
+            values,
+            ncols
+        );
+    }
+
     Rcpp::List out = Rcpp::List::create(
         Rcpp::_["str"] = R_NilValue
     );
@@ -624,7 +775,7 @@ Rcpp::List write_single_label_numeric_to_str_R
         ncols,
         nclasses,
         ignore_zero_valued,
-        sort_indices,
+        false,
         text_is_base1,
         add_header,
         decimal_places
@@ -653,6 +804,16 @@ Rcpp::List write_single_label_integer_to_str_R
     const int decimal_places
 )
 {
+    if (sort_indices)
+    {
+        sort_sparse_indices_known_ncol(
+            indptr,
+            indices,
+            values,
+            ncols
+        );
+    }
+
     Rcpp::List out = Rcpp::List::create(
         Rcpp::_["str"] = R_NilValue
     );
@@ -672,7 +833,7 @@ Rcpp::List write_single_label_integer_to_str_R
         ncols,
         nclasses,
         ignore_zero_valued,
-        sort_indices,
+        false,
         text_is_base1,
         add_header,
         decimal_places
